using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace CongressXMLTool
{
    #region Congress Event Data definition
    /// <summary>
    /// holds the data of one particular category
    /// </summary>
    public class CongressEvent
    {
        public Int32 Id;
        public Int32 DayIndex;
        public DateTime StartDate;
        public DateTime StartTime;
        public TimeSpan duration;
        public String Room;
        public String Title;
        public String Subtitle;
        public String track;
        public String language;
        public String abstract_text;
        public String description;

        public List<String> Persons;

        public List<String> Links;

        public CongressEvent()
        {
            Persons = new List<String>();
            Links = new List<String>();
        }
    }
    #endregion

    #region Sorter
    public static class Sorter
    {
        public static List<CongressEvent> SortID(List<CongressEvent> unsortedEvents, bool ascending)
        {

            List<CongressEvent> sortedEvents = new List<CongressEvent>();

            // now do some InsertionSort style sorting
            foreach (CongressEvent event_ in unsortedEvents)
            {
                if (sortedEvents.Count == 0)
                    sortedEvents.Insert(0, event_);
                else
                {
                    bool inserted = false;
                    // find a place to insert it..
                    for (int i = 0; i < sortedEvents.Count; i++)
                    {
                        if (ascending)
                        {
                            if (sortedEvents[i].Id > event_.Id)
                            {
                                inserted = true;
                                sortedEvents.Insert(i, event_);
                                break;
                            }
                        }
                        else
                        {
                            if (sortedEvents[i].Id < event_.Id)
                            {
                                inserted = true;
                                sortedEvents.Insert(i, event_);
                                break;
                            }
                        }
                    }
                    if (!inserted)
                        sortedEvents.Add(event_);
                }
            }

            return sortedEvents;
        }
    }
    #endregion

    class Program
    {
        public static List<CongressEvent> CongressEvents = new List<CongressEvent>();

        static void Main(string[] args)
        {
            #region Load Conference XML
            XmlTextReader reader = new XmlTextReader("schedule.en.xml");

            CongressEvent currentEvent = new CongressEvent();

            DateTime currentDay = DateTime.Now;
            bool addoneday = false;
            bool raushier = false;

            while (reader.Read())
            {
                switch (reader.NodeType)
                {
                    case XmlNodeType.EndElement:
                        switch (reader.Name)
                        {
                            case "conference":
                                raushier = true;
                                break;
                        }
                        break;
                }
                if (raushier) break;
            }

            while (reader.Read())
            {
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element: // Der Knoten ist ein Event
                        switch (reader.Name)
                        {
                            #region Nodes
                            case "day":
                                DateTime.TryParse(reader["date"], out currentDay);
                                //currentDay = DateTime.Now.Date;
                                break;
                            case "event":
                                currentEvent = new CongressEvent();
                                currentEvent.Id = Convert.ToInt32(reader["id"]);
                                currentEvent.StartDate = currentDay;
                                break;
                            case "start":
                                reader.Read();
                                DateTime.TryParse(reader.Value, out currentEvent.StartTime);
                                if (currentEvent.StartTime.Hour == 0)
                                {
                                    addoneday = true;
                                }
                                break;
                            case "duration":
                                reader.Read();
                                TimeSpan.TryParse(reader.Value, out currentEvent.duration);
                                break;
                            case "room":
                                reader.Read();
                                currentEvent.Room = reader.Value;
                                break;
                            case "title":
                                reader.Read();
                                currentEvent.Title = reader.Value;
                                break;
                            case "subtitle":
                                reader.Read();
                                currentEvent.Subtitle = reader.Value;
                                break;
                            case "track":
                                reader.Read();
                                currentEvent.track = reader.Value;
                                break;
                            case "language":
                                reader.Read();
                                currentEvent.language = reader.Value;
                                break;
                            #endregion
                        }
                        break;
                    case XmlNodeType.Text: //Anzeigen des Textes in jedem Element.
                        //Console.WriteLine(reader.Value);
                        break;
                    case XmlNodeType.EndElement: //Anzeigen des Endes des Elements.
                        switch (reader.Name)
                        {
                            case "event":
                                if (addoneday)
                                {
                                    currentEvent.StartDate = currentEvent.StartDate.AddDays(1);
                                    addoneday = false;
                                }
                                CongressEvents.Add(currentEvent);
                                break;
                        }
                        break;
                }
            }
            #endregion

            CongressEvents = Sorter.SortID(CongressEvents, true);

            foreach (CongressEvent event_ in CongressEvents)
            {
                Console.WriteLine(event_.Id + "-" + event_.Title.Replace(':', '_').Replace(' ', '_').Replace(',', '_').Replace('.', '_').Replace('?', '_').Replace('/', '_').Replace('\\', '_').Replace('\'', '_').Replace('\"', '_').Replace('&', '_').Replace('>','_').Replace('<','_'));
            }
            
        }
    }
}
